<?php

abstract class Slicer{
    
    public $exe = "";
    public $args = array();
    public $layerHeight = 0.5;
    public $infillPercentage = 50;
    public $printSpeed = 30;
    public $pricePerHour = 2;
    public $pricePerGram = 0.2;
    
    public $gCodeFile = "";
    public $stlFile = "";
    public $javaExe = "C:/Program Files/Java/jdk1.7.0_45/bin/java.exe";
    public $gCodeInfoJar = "c:/xampp/htdocs/GCodeInfoV98.jar";
    
    abstract function buildArgs();
    abstract function slice();
    
    function checkArgs(){
        if (file_exists($this->stlFile) == false)
            trigger_error("STL file not found at ".$this->stlFile, E_USER_ERROR);
        if (is_file($this->stlFile) == false)
            trigger_error("STL path is not a file, ".$this->stlFile, E_USER_ERROR);
        if ($this->infillPercentage < 0 || $this->infillPercentage > 100)
            trigger_error("Infill percentage out of bounds: ".$this->infillPercentage.", (0 to 100) ".$this->stlFile, E_USER_ERROR);
    }
}
  
class Cura extends Slicer{
    
    function __construct(){
        $this->exe = "c:/xampp/htdocs/Cura/CuraEngine.exe";
    }
    
    function buildArgs(){
        $this->checkArgs();
        $this->args[] = "-v";
        $this->args[] = "-s layerThickness=".($this->layerHeight*1000);
        
        $this->args[] = "-s sparseInfillLineDistance=".(100*($this->layerHeight*1000)/$this->infillPercentage);
        $this->args[] = "-s printSpeed=".$this->printSpeed;
        $this->args[] = "-s infillSpeed=".$this->printSpeed;
        $this->args[] = "-o ".$this->gCodeFile." ".$this->stlFile;
    }
    
    function slice(){
        $this->buildArgs();
        
        $command = $this->exe." ".implode(" ", $this->args);
        //echo $command;
        exec($command, $output, $return);
        
        $printTime = $filamentUsed = 0;
        
        foreach($output as $line){
            if (substr($line, 0, 4) == "Fail"){
                echo "<pre>".print_r($output,true)."</pre>"; 
                trigger_error("Cura failed, read output above", E_USER_ERROR);   
            }
            else if (substr($line, 0, 12) == "Print time: ")
                $printTime = substr($line, 12);
            else if (substr($line, 0, 10) == "Filament: ")
                $filamentUsed = substr($line, 10);
        }
        
        //echo "<pre>".print_r($output,true)."</pre>"; 
        //echo $printTime . " " . $filamentUsed;
        
        if ($printTime != '' && $filamentUsed != '' && $printTime != 0 && $filamentUsed != 0)
            return array("printTime"=>$printTime,"filamentUsed"=>$filamentUsed);
        else
            trigger_error("Impossible results detected, print time: ".$printTime.", filament used: ".$filamentUsed, E_USER_ERROR);
    }
}

class KISSlicer extends Slicer{
    //KISSlicer requires a config file in addition to command-line arguments
    function __construct(){
        $this->exe = "c:/xampp/htdocs/KISSlicer/KISSlicer64.exe";
        $this->configFile = "c:/xampp/htdocs/KISSlicer/_styles.ini"; 
    }
    
    function buildArgs(){
        //$this->checkArgs();
        $this->args[] = $this->stlFile;
        $this->args[] = "-o ".$this->gCodeFile;
        
        $modifiedLines = array();
        $i=0;
        $fp = fopen($this->configFile, 'r');
        
        while(feof($fp) == false){
            $line = str_replace(array("\r\n","\n"),"", fgets($fp));
            if (stripos($line, 'infill_density_denominator=') !== FALSE){
                $modifiedLines[$i] = 'infill_density_denominator=';
                $allowedInfills = array(50, 33.3, 25, 20, 16.7, 12.5, 10, 5, 2.5, 0);
                
                foreach($allowedInfills as $key => $value){
                    if ($this->infillPercentage >= $value){
                        $modifiedLines[$i] .= $key;
                        break;
                    }
                }
            }
            else if (stripos($line, 'layer_thickness_mm=') !== FALSE)
                $modifiedLines[$i] = 'layer_thickness_mm='.$this->layerHeight;
            else if (stripos($line, 'skin_thickness_mm=') !== FALSE)
                $modifiedLines[$i] = 'skin_thickness_mm='.$this->layerHeight;
            else
                $modifiedLines[$i] = $line;
            
            $i++;
        }
        
        fclose($fp);
        file_put_contents($this->configFile, implode(PHP_EOL, $modifiedLines));
    }
    
    function slice(){
        $this->buildArgs();
        
        $command = $this->exe." ".implode(" ", $this->args);
        //echo $command;
        exec($command, $output, $return);
        //echo $return;
        
        $printTime = $filamentUsed = '';
        
        $command2 = "\"".$this->javaExe."\" -jar \"".$this->gCodeInfoJar."\" m \"".$this->gCodeFile."\" 2>&1";
        //echo $command2;
        exec($command2,$output2,$return2);
        //echo $return2;
        
        echo "<pre>".print_r($output2,true)."</pre>"; 
        
        foreach($output2 as $line){
            if (strpos($line,'Weight: ') !== false){
                $filamentUsed = str_replace('Weight: ','',substr($line,0,-1));
            }
            else if (strpos($line,'Overall Time (w/ Acceleration):') !== false){
                $printTime = str_replace("Overall Time (w/ Acceleration):    ","",$line);
                $printTime = substr($printTime, strpos($printTime,'(')+1, -4);
            }
        }
        
        if ($printTime != '' && $filamentUsed != '' && $printTime != 0 && $filamentUsed != 0)
            return array("printTime"=>$printTime,"filamentUsed"=>$filamentUsed);
        else
            trigger_error("Impossible results detected, print time: ".$printTime.", filament used: ".$filamentUsed, E_USER_ERROR);
        
    }
}